package com.docus.webservice.handler;


import com.alibaba.fastjson.JSON;
import com.docus.webservice.dto.CollectList;
import com.docus.webservice.dto.SelectColumn;
import com.docus.webservice.entity.AfInterfaceCollect;
import com.docus.webservice.entity.AfInterfaceCollectSub;
import com.docus.webservice.entity.MrReportError;

import com.docus.webservice.entity.TBasic;
import com.docus.webservice.enums.Codes;
import com.docus.webservice.mapper.AfInterfaceCollectMapper;
import com.docus.webservice.service.IMrReportErrorService;
import com.docus.webservice.service.ITBasicService;
import com.docus.webservice.utils.*;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import javax.jws.WebService;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URISyntaxException;
import java.util.*;

@WebService
public class TBasicWebService implements ITBasicWebService {

    @Autowired
    private ITBasicService itBasicService;

    @Autowired
    private IMrReportErrorService mrReportErrorService;

    @Autowired
    private AfInterfaceCollectMapper afInterfaceCollectMapper;

    private Logger log = LogManager.getLogger(TBasicWebService.class);

    /**
     * 获取jar包所在位置
     *
     * @return
     */
    private String CurrentPath() {
        File dir = new File(".");
        String currentpath = "";
        try {
            currentpath = dir.getCanonicalPath();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return currentpath;
    }

    /**
     * 记录日志
     */
    private void insertLog(String xml) {
        MrReportError mrReportError = new MrReportError();
        mrReportError.setCreateTime(new Date());
        mrReportError.setReportType(Integer.parseInt(Codes.ERROR_CODE.getMessage()));
        mrReportError.setXml(xml);
        mrReportErrorService.save(mrReportError);
    }

    @Override
    public String OUT_HOS_REG(String tbasic) {
        log.info("住院成员信息新增xml" + tbasic);
        try {
            //解析xml
            HashMap<String, Object> tBasicMap = new HashMap<>();
            HashMap<String, Object> tBasicSubMap = new HashMap<>();
            parseXml(tbasic, tBasicMap, tBasicSubMap);
            //处理业务
            TBasic tBasic;
            if (tBasicMap != null && tBasicMap.size() > 0) {
                tBasic = JSON.parseObject(JSON.toJSONString(tBasicMap), TBasic.class);
                tBasic.setCreateTime(new Date());
                itBasicService.savaAndSub(tBasic, tBasicSubMap);
                this.saveAndCall(tBasic);
            } else {
                insertLog(tbasic);
                log.info("请提供完整的xml");
                return ResultUtils.fail("请提供完整的xml");
            }
        } catch (RuntimeException e) {
            insertLog(tbasic);
            log.error("OUT_HOS_REG方法中错误信息:" + e.getMessage());
            return ResultUtils.fail(e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            log.error("OUT_HOS_REG方法中错误信息:" + e.getMessage());
            return ResultUtils.fail();
        }
        log.info("返回结果:" + ResultUtils.success().asXML());
        return ResultUtils.success().asXML();
    }

    /**
     * 解析xml
     *
     * @param tbasic
     * @param tBasicMap
     * @param tBasicSubMap
     */
    private void parseXml(String tbasic, HashMap<String, Object> tBasicMap, HashMap<String, Object> tBasicSubMap) {
        XmlUtils xmlUtils = null;
        log.info("开始解析xml");
        try {
            xmlUtils = new XmlUtils(new ByteArrayInputStream(tbasic.getBytes("UTF-8")));
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        //解析json映射文件
        String json = JsonUtils.readJsonFile(CurrentPath() + Codes.JSON_ADDRESS.getMessage());
        Map jsonMap = JSON.parseObject(json, Map.class);

        List<SelectColumn> selectColumns = JSON.parseArray(String.valueOf(jsonMap.get(Codes.SELECT_COLUMNS.getMessage())), SelectColumn.class);
        String value;
        for (SelectColumn selectColumn : selectColumns) {
            try {
                value = xmlUtils.getElementText(selectColumn.getQualityColumn()) == null ? null : String.valueOf(xmlUtils.getElementText(selectColumn.getQualityColumn()));
            } catch (RuntimeException e) {
                String message = e.getMessage();
                log.error("解析xml出现:" + message);
                throw new RuntimeException(message);
            }
            if (StringUtils.isBlank(value)) {
                if (selectColumn.getByNull() == 0) {
                    insertLog(tbasic);
                    log.error(selectColumn.getQualityColumn() + " 字段不能为空值");
                    throw new RuntimeException(selectColumn.getQualityColumn() + " 字段不能为空值");
                } else {
                    continue;
                }
            } else {
                if (selectColumn.getTableName().equals("t_basic")) {
                    tBasicMap.put(selectColumn.getColumnName(), value);
                }
                if (selectColumn.getTableName().equals("t_basic_sub")) {
                    tBasicSubMap.put(selectColumn.getColumnName(), value);
                }
            }
        }
    }

    @Override
    public String OUT_HOS_CAN(String tbasic) {
        log.info("住院成员信息修改xml" + tbasic);
        try {
            HashMap<String, Object> tBasicMap = new HashMap<>();
            HashMap<String, Object> tBasicSubMap = new HashMap<>();
            parseXml(tbasic, tBasicMap, tBasicSubMap);
            //处理业务
            TBasic tBasic;
            if (tBasicMap != null && tBasicMap.size() > 0) {
                tBasic = JSON.parseObject(JSON.toJSONString(tBasicMap), TBasic.class);
                if (StringUtils.isBlank(tBasic.getEmpId())) {
                    insertLog(tbasic);
                    log.info("emp_id字段不能为空值");
                    return ResultUtils.fail("emp_id字段不能为空值");
                }
                tBasic.setUpdateTime(new Date());
                try {
                    itBasicService.updateAndSub(tBasic, tBasicSubMap);
                    this.saveAndCall(tBasic);
                } catch (RuntimeException e) {
                    insertLog(tbasic);
                    e.printStackTrace();
                    log.error("OUT_HOS_CAN方法中错误信息:" + e.getMessage());
                    return ResultUtils.fail(e.getMessage());
                }
            } else {
                insertLog(tbasic);
                return ResultUtils.fail("请提供完整的xml");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("OUT_HOS_CAN方法中错误信息:" + e.getMessage());
            return ResultUtils.fail();
        }
        log.info("OUT_HOS_CAN方法运行结果:" + ResultUtils.success().asXML());
        return ResultUtils.success().asXML();
    }


    public void saveAndCall(TBasic tBasic) throws URISyntaxException {
        log.info("病案对象" + tBasic.toString());
        //解析json映射文件
        String json = JsonUtils.readJsonFile(CurrentPath() + Codes.JSON_COLLECTLIST.getMessage());
        if (StringUtils.isNotBlank(json)) {
            Map jsonMap = JSON.parseObject(json, Map.class);
            //任务数
            Integer task_count = JSON.parseObject(String.valueOf(jsonMap.get(Codes.JSON_TASK_COUNT.getMessage())), Integer.class);

            List<CollectList> collectLists = JSON.parseArray(String.valueOf(jsonMap.get(Codes.JSON_COLLECTLIST_ROOT.getMessage())), CollectList.class);

            //添加省中医病案采集
            SnowflakeIdWorker idWorker = new SnowflakeIdWorker(0, 0);
            AfInterfaceCollect afc = new AfInterfaceCollect();
            Long afcId = idWorker.nextId();
            afc.setId(afcId);
            afc.setJzh(tBasic.getJzh());
            afc.setTaskCount(task_count);
            afc.setCreateTime(new Date());
            int i = afInterfaceCollectMapper.addAfInterfaceCollect(afc);
            if (i < 0) {
                log.info("省中医病案采集表添加信息失败！");
            }
            //添加省中医病案采集-子任务
            List<AfInterfaceCollectSub> list = new ArrayList<>();
            for (CollectList collectList : collectLists) {
                AfInterfaceCollectSub afInterfaceCollectSub = new AfInterfaceCollectSub();
                Long afcsId = idWorker.nextId();
                afInterfaceCollectSub.setId(afcsId);
                afInterfaceCollectSub.setAfInterfaceCollectId(afcId);
                afInterfaceCollectSub.setCollectsysCode(collectList.getCollectsys_code());
                afInterfaceCollectSub.setJzh(tBasic.getJzh());
                collectList.setId(afcsId);
                list.add(afInterfaceCollectSub);
            }
            int i1 = afInterfaceCollectMapper.addAfInterfaceCollectSub(list);
            if (i1 < 0) {
                log.info("省中医病案采集-子任务失败！");
            }
            for (CollectList collectList : collectLists) {
                //调用http发送请求
                this.sendHttp(collectList, tBasic.getJzh());
            }
        }
    }

    private String sendHttp(CollectList collectList, String jzh) throws URISyntaxException {
        Map<String, String> params = new HashMap<>();
        params.put("empId", jzh);
        params.put("collectSubId", String.valueOf(collectList.getId()));
        return HttpUtils.get(collectList.getRequestUrl(), params);
    }


}
